/*! This file exists only for testing a Minify installation. Its content is not used.
 *
 * http://example.org/min/f=min/quick-test.less
 */

// LESS import statement shares syntax with the CSS import statement.
// If the file being imported ends in a .less extension, or no extension, then it is treated as a LESS
// import. Otherwise it is left alone and outputted directly.
// http://leafo.net/lessphp/docs/#import
@import "quick-test.css";

@import "quick-testinc.less";